# -*- coding: utf-8 -*-
# Standard Libraries
from enum import StrEnum


class Collections(StrEnum):
    AMAZONIA1_WFI_L2_DN = "AMAZONIA1_WFI_L2_DN"
    AMAZONIA1_WFI_L4_DN = "AMAZONIA1_WFI_L4_DN"
    CBERS4A_MUX_L2_DN = "CBERS4A_MUX_L2_DN"
    CBERS4A_MUX_L4_DN = "CBERS4A_MUX_L4_DN"
    CBERS4A_WFI_L2_DN = "CBERS4A_WFI_L2_DN"
    CBERS4A_WFI_L4_DN = "CBERS4A_WFI_L4_DN"
    CBERS4A_WPM_L2_DN = "CBERS4A_WPM_L2_DN"
    CBERS4A_WPM_L4_DN = "CBERS4A_WPM_L4_DN"
    CBERS4_AWFI_L2_DN = "CBERS4_AWFI_L2_DN"
    CBERS4_AWFI_L4_DN = "CBERS4_AWFI_L4_DN"
    CBERS4_MUX_L2_DN = "CBERS4_MUX_L2_DN"
    CBERS4_MUX_L4_DN = "CBERS4_MUX_L4_DN"
    CBERS4_PAN10M_L2_DN = "CBERS4_PAN10M_L2_DN"
    CBERS4_PAN10M_L4_DN = "CBERS4_PAN10M_L4_DN"
    CBERS4_PAN5M_L2_DN = "CBERS4_PAN5M_L2_DN"
    CBERS4_PAN5M_L4_DN = "CBERS4_PAN5M_L4_DN"

    def __repr__(self) -> str:
        return str.__repr__(self.value)
